package Dates;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.StringTokenizer;


/**
 * 
 * @author Pierre Courtieu
 *
 * <p>
 * Cette classe implémente un menu avec des opérations très simples sur une liste de
 * dates <i>courantes</i>.
 * <ul>
 * <li> ajout d'une date à la liste
 * <li> vider la liste des dates
 * <li> déterminer le dernier jour du mois d'une date de la liste
 * <li> etc
 * </ul></p>
 * <p>
 * L'implémentation est volontairement non objet. Les dates sont représentées par trois
 * entiers et les méthodes travaillant sur une date prennent 3 entiers en paramètres. Voir
 * par exemple la méthode {@code dateToString(int a, int m int j)}.</p>
 * <p>
 * De la même façon la liste des dates courantes est représentée par trois tableaux d'entiers
 * ({@code ArrayList<Integer>}), un pour les années, un pour les mois et un pour
 * les jours. Ces tableaux sont passés en paramètres à toutes les méthodes ayant besoin
 * d'accéder à la liste des dates. Voir par exemple la méthode {@code ajouteDate}.</p>
 * 
 * <p>Exemple, si les trois tableaux contiennent ceci:
 * <pre><code>
 * annees = { 2001, 2002, 2002 }
 * mois =   { 12  , 6   , 3 }
 * jours =  { 23  , 18  , 20 }
 * </code></pre>
 * <p>alors les trois dates actuellement stockées sont:
 * <ol>
 * <li> 23/12/2001
 * <li> 18/06/2002
 * <li> 20/3/2002
 * </ol>
 * <p>Ce code ganerait en clarté si une date était représentée par une valeur de type Date, et non
 * par trois valeurs de type int stockées dans différents tableaux. Et la liste des date serait
 * donc un simple tableau de dates.
 * 
 * <p>Votre travail consiste à modifier cette implémentation de la manière suivante:
 * 
 * <ol>
 * <li> Créer une classe Date. Un objet de la classe {@code Date} doit représenter une date.
 *    Les opérations relatives aux dates seront implémentées par des méthodes non statiques.
 *    On pourra alors par exemple remplacer un code comme celui-ci:
 *    <pre><code>
 *    int a = 2012, m = 7, j=18; // création de trois entier
 *    System.out.println(dateToString(a,m,j)) // Appel à une méthode statique sur trois entier
 *    </code></pre>
 *    par
 *    <pre><code>
 *    Date d = new Date(2012,7,18); // création d'une date
 *    System.out.println(d.toString()); // appel à la méthode toString de objet d
 *    </code></pre>
 * <li> coder les opérations concernant les dates
 * <li> Dans une classe {@code DateAppli} reprogrammer le programme principal en utilisant
 *    la classe {@code Date}, avec une seule variable de type {@code ArrayList<Date>}
 *    plutôt que 3 tableaux d'entiers. Les menus seront aussi programmé dans cette classe.
 * <li> Enrichissez le menu avec une opération "trouver les dates extrêmes" de la liste courante.
 * </ol>

 *
 *  Structure de la classe Date:
 *  
 *    <pre><code>
	   public class Date {
	   	  int jour ;
	   	  int mois;
	   	  int annee;
	   	  public Date(int a, int m, int j){
	   	     this.jour = j;
	   	     this.mois = m;
	   	     this.annee = a;
	   }
	   public String toString(){
	     ...
	   }
	   public dernierJourDuMois(){
	      ...
	   }
	   ...
 *    </code></pre>
 *
 *
 *
 *
 */
public class Main {

	/** Le tableau des derniersjours des mois (année bissextiles non gérées)*/
	static int [] longMois      = { -1 , 31,28,31,30,31,30,31,31,30,31,30,31}; 
	
	/**
	 * Affichage d'une date.
	 * @param a année
	 * @param m mois
	 * @param j jour
	 * @return la chaîne de caractère représentant la date.
	 */
	static String dateToString(int a, int m,int j){
		return j+"/"+ m + "/" + a;
	}
	
	/**
	 * retourne le dernier jour du mois
	 * @param m le mois
	 * @return le numéro du dernier jour du mois m.
	 * @throws ArrayIndexOutOfBoundsException si m n'est pas entre 1 et 12.
	 */
	static int dernierJourDuMois(int m) {
		return longMois[m];
	}

	/**
	 * Test de validité d'une date.
	 * @param a année
	 * @param m mois
	 * @param j jour
	 * @return true si la date est valide (années bissextiles non prises en compte)
	 */
	static boolean dateIsValide(int a, int m, int j) {
		if (m<1 || m >12) return false;
		if (j<1 || j > dernierJourDuMois(m)) return false;
		return true;
	}
	
	/**
	 * Affiche toutes les dates décrites par les trois tableaux, chacune précédées d'un numéro.
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 */
	static void afficheLesDates(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours) {
		for(int i = 0 ; i < annees.size() ; i++) {
			String date = dateToString(annees.get(i),mois.get(i),jours.get(i));
			System.out.println((i+1) + ". " + date);
		}
	}
	
	/**
	 * Ajoute une date dans les trois tableaux anne, mois et jours. Pas de vérification de validité de la date.
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 * @param a l'année de la date à ajouter
	 * @param m le mois de la date à ajouter
	 * @param j le jour de la date à ajouter
	 */
	static void ajouteDate(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours, int a, int m, int j) {
		annees.add(a);
		mois.add(m);
		jours.add(j);
	}

	/**
	 * Affiche le menu principal, puis demande son choix à l'utilisateur (en recommençant si nécessaire) et le retourne. 
	 * @return le choix de l'utilisateur.
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 */
	static int mainMenu(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours) {
		afficheLesDates(annees,mois,jours); // Affiche les date numérotées
		System.out.println("Que vouslez vous faire?\n"
				+ " 0- Quitter.\n"
				+ " 1- ajouter des dates.\n"
				+ " 2- supprimer toutes les dates.\n"
				+ " 3- afficher les dates extrêmes.\n"
				+ " 4- savoir quel est le dernier jour du mois d'une date. ");
		
		return Terminal.demandeEntier(0, 4); // demande un entier entre 0 et 4
	}
	
	/**
	 * Affiche le sous-menu de l'ajout d'une date, et ajoute la date tapée si elle est valide. Ne fait rien sinon.
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 */
	static void menuAjouteDate(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours) {
		System.out.println("tapez une date.");
		try {
			int t[] = Terminal.lireDate();
			if (dateIsValide(t[0],t[1],t[2])) {	ajouteDate(annees,mois,jours,t[0],t[1],t[2]); }
			else { System.out.println("Date invalide. Opération annulée."); }
		} catch (TerminalException e) { System.out.println("Erreur de saisie."); }
	}
	
	/**
	 * Affiche les 2 dates extrêmes parmis toutes les date décrites dans les trois tableaux .
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 */
	static void afficheExtremes(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours) {
		throw new RuntimeException("Pas encore implanté");
	}

	/**
	 * Affiche le sous-menu du "dernier jour du mois". Demande de désigner une date parmis celles affichées, puis affiche le dernier jour du mois de la date désignée.
	 * @param annees les années des dates, dans l'ordre
	 * @param mois les mois des dates, dans l'ordre
	 * @param jours les jours des dates dans l'ordre
	 */
	static void menuDernierDate(ArrayList<Integer> annees, ArrayList<Integer> mois, ArrayList<Integer> jours)  {
		afficheLesDates(annees,mois,jours);
		System.out.println("Choisissez une date: ");
		int num = -1;
		while (num<0) {
			try {
				int answer = Terminal.lireInt();
				if (answer>=1 && answer <= annees.size()) {
					num=answer-1;
				} else { num = -1; }
			} catch (TerminalException e) {} // skip one character to avoid looping on the bad input
		}
		System.out.println("le dernier jour du mois de la date "
				+ (dateToString(annees.get(num),mois.get(num),jours.get(num)))
				+" est: " + dateToString(annees.get(num),mois.get(num),dernierJourDuMois(mois.get(num))));
	}

	/** Programme pprincipal, qui effectue le menu principal en boucle.
	 * @param args paramètre inutilisé */
	public static void main(String[] args) {
		ArrayList<Integer> annees = new ArrayList<Integer>();
		ArrayList<Integer> mois = new ArrayList<Integer>();
		ArrayList<Integer> jours = new ArrayList<Integer>();

		// On met quelques dates au départ.
		ajouteDate(annees, mois, jours, 2022, 6, 1);
		ajouteDate(annees, mois, jours, 2022, 9, 2);
		ajouteDate(annees, mois, jours, 2022, 11, 7);
		ajouteDate(annees, mois, jours, 2022, 4, 3);
		int choix;
		boolean sortir= false;
		while (!sortir) {
			System.out.println("\n");
			choix = mainMenu(annees, mois, jours);
			switch (choix) {
			case 1:
				menuAjouteDate(annees, mois, jours);
				break;
			case 2:
				annees.removeAll(annees);
				mois.removeAll(mois);
				jours.removeAll(jours);
				break;
			case 3:
				afficheExtremes(annees, mois, jours);
				break;
			case 4:
				menuDernierDate(annees, mois, jours);
				break;
			case 0: sortir = true;
				break;
			default:
				break;
			}
		}
	}

}
